@@ -15,6 +15,14 @@
 #  limitations under the License.
 
 from PySide import QtGui, QtCore
+
+DEBUG_LEVEL = 0
+
+def debug(level, *args):
+    if level <= DEBUG_LEVEL:
+        for s in args:
+            print s,
+        print
 import urllib2, sudsds
 import re
 
@@ -37,7 +45,7 @@ class LabelWordWrap(Label):
                                      QtCore.Qt.TextSelectableByMouse)
 
 class PlusButton(QtGui.QPushButton):
-    def __init__(self, parent, ClientObj, method):
+    def __init__(self, parent, ClientObj, method, disable_method=None):
         QtGui.QPushButton.__init__(self, parent)
         try:
             self.setIcon(QtGui.QIcon.fromTheme('list-add'))
@@ -47,6 +55,8 @@ class PlusButton(QtGui.QPushButton):
         self.ClientObj = ClientObj
         self.method = method
         self.setFixedWidth(30)
+        if disable_method:
+            self.clicked.connect(disable_method)
         self.clicked.connect(self.call_method)
 
     def call_method(self):
@@ -86,6 +96,7 @@ class MainFrameRes(QtGui.QWidget):
         self.change_offset = change_offset
         self.show_last = True
         self.max_val = None
+        self.buttons = []
 
         if self.meth_result.type == "pid":
             self.cur_pid = self.meth_result.message
@@ -106,9 +117,9 @@ class MainFrameRes(QtGui.QWidget):
         else:
             self._parent.methodname_wgt.setMethodName(method_name)
         parent.main_frame.viewport().setObjectName('main_frame')
-        parent.main_frame.viewport().setStyleSheet("#main_frame "
-                                            "{background-color: #FFFFFF;}")
-                                            #"{background-color: #E4E1E0;}")
+        #parent.main_frame.viewport().setStyleSheet("#main_frame "
+        #                                    "{background-color: #FFFFFF;}")
+        #                                    #"{background-color: #E4E1E0;}")
         self.setObjectName("Frame_Res")
         self.setStyleSheet("#Frame_Res {background-color: transparent;}")
         #self.setStyleSheet("#Frame_Res {background-color: green;}")
@@ -119,11 +130,13 @@ class MainFrameRes(QtGui.QWidget):
         self.timer.start(100)
         self.valSignal.connect(self.show_bottom)
 
+
     def initUI(self):
         #self.setSizePolicy(QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Minimum)
         self.updateGeometry()
 
     def show_bottom(self):
+        debug(20, "SHOW BOTTOM PAGE")
         max_val = self._parent.main_frame.verticalScrollBar().maximum()
         current_val = \
             self._parent.main_frame.verticalScrollBar().sliderPosition()
@@ -151,54 +164,63 @@ class MainFrameRes(QtGui.QWidget):
         if not table.body:
             table.body = [[]]
         self.show_normal(item)
+        self.mytable = None
 
         if self.method_name in self.ClientObj.param_objects:
             if len(table.body[0]) >= self.ClientObj.count_row_res_table or \
                     self.ClientObj.param_objects[self.method_name]['offset']:
 
-                if table.onClick:
-                    self.add_offset_buttons(len(table.body[0]),table.addAction)
+                if table.onClick and table.records != "0":
+                    self.add_offset_buttons(len(table.body[0]),
+                                            table.addAction,
+                                            table.records)
                     add_offset_flag = True
 
         if table.addAction and not add_offset_flag:
-            self.work_layout._addWidget(PlusButton(self, self.ClientObj, \
-                                        table.addAction))
-        if table.onClick:
+            plus_but = PlusButton(self, self.ClientObj, table.addAction,
+                                  self.disable_buttons)
+            self.buttons.append(plus_but)
+            self.work_layout._addWidget(plus_but)
+        if table.onClick and table.body[0]:
             self.mytable = SelectedTableWidget(len(table.body[0]), \
                                                len(table.head[0]), self)
-
-        if not hasattr (self, 'mytable'):
+        if not self.mytable:
             if table.body:
                 len_body = len(table.body[0])
             else:
-                len_body = 1
+                len_body = 0
             if table.head:
                 len_head = len(table.head[0])
             else:
                 len_head = 1
-            self.mytable = SelectedTableWidget(len_body, len_head, self,
-                                                   readonly = True)
-        list_head = table.head[0]
-        self.mytable.setHorizontalHeaderLabels(list_head)
-        for line in range(len(table.body[0])):
-            for i in range(len(table.body[0][line][0])):
-                tablewidgetitem = QtGui.QTableWidgetItem \
-                                                    (table.body[0][line][0][i])
-                self.mytable.setItem(line, i, tablewidgetitem)
-#         set writeonly
-        self.mytable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
+            if len_body:
+                self.mytable = SelectedTableWidget(len_body, len_head, self,
+                                                   readonly=True)
+        if self.mytable:
+            list_head = table.head[0]
+            self.mytable.setHorizontalHeaderLabels(list_head)
+            for line in range(len(table.body[0])):
+                for i in range(len(table.body[0][line][0])):
+                    tablewidgetitem = QtGui.QTableWidgetItem(
+                        table.body[0][line][0][i])
+                    #tablewidgetitem.setTextAlignment(
+                    #    QtCore.Qt.Align | QtCore.Qt.AlignLeft)
+                    self.mytable.setItem(line, i, tablewidgetitem)
+            # set writeonly
+            self.mytable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
 
-        if table.onClick:
-            self.mytable.itemClicked.connect(self.row_activate(table))
+            if table.onClick:
+                self.mytable.itemClicked.connect(self.row_activate(table))
 
-        self.mytable.setColor()
-        minimum = QtGui.QSizePolicy.Minimum
-        self.mytable.setSizePolicy(minimum, minimum)
-        self.mytable.verticalHeader().hide()
-        self.mytable.horizontalHeader().resizeSections \
-                                (QtGui.QHeaderView.ResizeMode.ResizeToContents)
-        self.work_layout._addWidget(self.mytable)
-        self.ClientObj.app.processEvents()
+            self.mytable.setColor()
+            minimum = QtGui.QSizePolicy.Minimum
+            self.mytable.setSizePolicy(minimum, minimum)
+            self.mytable.horizontalHeader().resizeSections(
+                QtGui.QHeaderView.ResizeMode.ResizeToContents)
+            self.work_layout._addWidget(self.mytable)
+            self.ClientObj.app.processEvents()
+            self.mytable.resizeRowsToContents()
+            self.mytable.resizeColumnsToContents()
 
         self.initUI()
 
@@ -207,19 +229,32 @@ class MainFrameRes(QtGui.QWidget):
             param_object = create_obj(self.client, table.onClick)
             for field in table.fields.string:
                 if field:
+                    if ":" in field:
+                        field, op, value = field.partition(":")
+                    else:
+                        value = None
                     if hasattr (param_object, field):
-                        param_object[field] = self.mytable.item(item.row(),0).\
+                        if value:
+                            param_object[field] = value
+                        else:
+                            param_object[field] = self.mytable.item(item.row(),0).\
                                               text()
+            #print param_object, table.step
+            param_object.CheckOnly = True
             # Calling method
             sid = int(self.ClientObj.sid)
             self.client.service[0][table.onClick](sid, param_object)
 
             # Calling View
             try:
-                view_params = get_view_params(self.client, \
-                                       table.onClick + '_view', step = 0)
-                view = self.client.service[0][table.onClick + '_view'] \
-                                    (sid, view_params)
+                self.client.service.clear_pid_cache(sid, self.cur_pid)
+            except:
+                pass
+            try:
+                view_params = get_view_params(self.client,
+                                       table.onClick + '_view', step = 0) #int(table.step) if table.step else 0)
+                view = self.client.service[0][table.onClick + '_view'](
+                    sid, view_params)
             except sudsds.MethodNotFound, e:
                 _print(e)
                 return
@@ -230,53 +265,117 @@ class MainFrameRes(QtGui.QWidget):
                 view_method = table.onClick
             self._parent.ClientObj._parent.setWindowTitle(view_method + \
                         ' - ' + self.ClientObj.Name)
-            self.ClientObj.param_objects[self.method_name]['offset'] = 0
-            self._parent.main_frame_view(view, table.onClick)
+            #self.ClientObj.param_objects[self.method_name]['offset'] = 0
+            self._parent.main_frame_view(
+                view, table.onClick,
+                new_step=table.step if table.step else None)
         return wrapper
 
-    def add_offset_buttons(self, len_body, addAction):
+    def disable_buttons(self):
+        debug(11, "DISABLE", self.buttons)
+        for button in self.buttons:
+            button.setDisabled(True)
+        self.timer.stop()
+
+    def add_offset_buttons(self, len_body, addAction, records):
         offset_buttons = QtGui.QWidget()
         layout = QtGui.QHBoxLayout(offset_buttons)
         layout.setAlignment(QtCore.Qt.AlignLeft)
 
+        offset = self.ClientObj.param_objects[self.method_name]['offset']
+        count_records = int(self.ClientObj.count_row_res_table)
+        debug(15, "DATA:", records, offset, count_records)
+        records = int(records) or 0
+        if records:
+            if offset + count_records >= records:
+                last = True
+            else:
+                last = False
+        else:
+            if len_body < self.ClientObj.count_row_res_table:
+                last = True
+            else:
+                last = False
+
+
+        begin_offset = QtGui.QPushButton(offset_buttons)
+        begin_offset.setIcon(QtGui.QIcon.fromTheme('go-first'))
+        begin_offset.setToolTip('First page')
+        begin_offset.clicked.connect(self.disable_buttons)
+        begin_offset.clicked.connect(self.call_method_with_offset(0, True))
+        begin_offset.setFixedWidth(30)
+        self.buttons.append(begin_offset)
+
+        layout.addWidget(begin_offset)
+
         minus_offset = QtGui.QPushButton(offset_buttons)
         minus_offset.setIcon(QtGui.QIcon.fromTheme('go-previous'))
         minus_offset.setToolTip('Previous page')
-        minus_offset.clicked.connect(self.call_method_with_offset \
-                    (-1*int(self.ClientObj.count_row_res_table)))
+        minus_offset.clicked.connect(self.disable_buttons)
+        minus_offset.clicked.connect(self.call_method_with_offset(
+            -count_records))
         minus_offset.setFixedWidth(30)
-
-        if not self.ClientObj.param_objects[self.method_name]['offset']:
-            minus_offset.setDisabled(True)
-
         layout.addWidget(minus_offset)
+        self.buttons.append(minus_offset)
+
+        if not offset:
+            minus_offset.setDisabled(True)
+            begin_offset.setDisabled(True)
+
 
         plus_offset = QtGui.QPushButton(offset_buttons)
         plus_offset.setIcon(QtGui.QIcon.fromTheme('go-next'))
         plus_offset.setToolTip('Next page')
-        plus_offset.clicked.connect(self.call_method_with_offset \
-                                    (int(self.ClientObj.count_row_res_table)))
-        plus_offset.setFixedWidth(30)        
-
-        if len_body < self.ClientObj.count_row_res_table:
-            plus_offset.setDisabled(True)
+        plus_offset.clicked.connect(self.disable_buttons)
+        plus_offset.clicked.connect(self.call_method_with_offset(
+            count_records))
+        plus_offset.setFixedWidth(30)
         layout.addWidget(plus_offset)
+        self.buttons.append(plus_offset)
+
+        last_offset = QtGui.QPushButton(offset_buttons)
+        last_offset.setIcon(QtGui.QIcon.fromTheme('go-last'))
+        last_offset.setToolTip('Last page')
+        last_offset.clicked.connect(self.disable_buttons)
+        last_offset.clicked.connect(self.call_method_with_offset(
+            0, False, records - count_records))
+        last_offset.setFixedWidth(30)
+        layout.addWidget(last_offset)
+        self.buttons.append(last_offset)
+
+        if last or not records:
+            last_offset.setDisabled(True)
+        if last:
+            plus_offset.setDisabled(True)
+
         if addAction:
-            layout.addWidget(PlusButton(self, self.ClientObj, addAction))
+            plus_but = PlusButton(self, self.ClientObj, addAction,
+                                  self.disable_buttons)
+            self.buttons.append(plus_but)
+            layout.addWidget(plus_but)
 
         self.work_layout._addWidget(offset_buttons)
 
-    def call_method_with_offset(self, offset):
+    def call_method_with_offset(self, offset, begin=False, reset=None):
         def wrapper():
-            self.ClientObj.param_objects[self.method_name]['offset'] += offset
-            if self.ClientObj.param_objects[self.method_name]['offset'] < 0:
+            debug(10, "CHANGE OFFSET FOR %s to %d" % (
+                self.method_name, offset))
+            if begin:
                 self.ClientObj.param_objects[self.method_name]['offset'] = 0
+            elif reset is not None:
+                self.ClientObj.param_objects[self.method_name]['offset'] = \
+                    reset
+            else:
+                self.ClientObj.param_objects[self.method_name]['offset'] += offset
+                if self.ClientObj.param_objects[self.method_name]['offset'] < 0:
+                    self.ClientObj.param_objects[self.method_name]['offset'] = 0
 
             # Calling View
+            debug(10, "CALL VIEW", self.method_name)
             sid = int(self.ClientObj.sid)
             try:
-                view_params = get_view_params(self.client, \
-                                       self.method_name + '_view', step = 0)
+                view_params = get_view_params(self.client,
+                                       self.method_name + '_view', step = None)
                 view = self.client.service[0][self.method_name + '_view'] \
                                                     (sid, view_params)
             except sudsds.MethodNotFound:
@@ -288,8 +387,7 @@ class MainFrameRes(QtGui.QWidget):
             view_method = self.method_name
             self._parent.ClientObj._parent.setWindowTitle(view_method + \
                         ' - ' + self.ClientObj.Name)
-            self._parent.main_frame_view(view, view_method, \
-                                         change_offset=True)
+            self._parent.main_frame_view(view, view_method, change_offset=True)
         return wrapper
 
     def show_message(self, message, mono=False, space=0):
@@ -344,7 +442,7 @@ class MainFrameRes(QtGui.QWidget):
             lbl_temp.setMaximumHeight(lbl_temp.minimumSizeHint().height())
             self.work_layout._addWidget(lbl_temp)
         self.initUI()
-        
+
     def show_group(self, item, *args):
         pass
 
@@ -354,7 +452,7 @@ class MainFrameRes(QtGui.QWidget):
 
         lbl_temp.setMinimumHeight(lbl_temp.sizeHint().height())
         self.work_layout._addWidget(lbl_temp)
-        
+
         self.initUI()
 
     def processTags(self, s):
@@ -417,6 +515,7 @@ class MainFrameRes(QtGui.QWidget):
         self.work_layout.hide_kill_button()
         self.closeDialogs()
         self.close_all_thread()
+
         if self.change_offset:
             self._parent.delete_MainFrameWgt()
             self._parent.MainFrameWgt = self
@@ -424,17 +523,18 @@ class MainFrameRes(QtGui.QWidget):
             self._parent.main_frame.setWidgetResizable(True)
             self._parent.methodname_wgt.show()
             self._parent.control_button.show()
-            self._parent.control_button.add_clear_cache_button(self.cur_pid)
-            self.initUI()
-            return 0
 
-        if hasattr (self, "mytable"):
-            if not self.mytable.readonly:
-                self.initUI()
-                return 0
+        #if hasattr (self, "mytable"):
+        #    if not self.mytable.readonly:
+        #        self.initUI()
+        #        return 0
 
+        #self._parent.control_button.add_clear_cache_button(self.cur_pid)
+        #self.initUI()
+        # TODO: возможно здесь не нужне return 1
         self._parent.control_button.add_clear_cache_button(self.cur_pid)
         self.initUI()
+        return 0
 
 #    def clear_pid_cache(self):
 #        self.clear_proc_cache_button.setDisabled(True)
@@ -595,11 +695,11 @@ class MainFrameRes(QtGui.QWidget):
         #                args = (sid, pid))
         #thread_messages.start()
         self.get_Frame(sid, pid)
-    
+
     def get_Frame(self, sid, pid, def_end_frame = 1):
         """ get all messages, until type is not endFrame (or Error) """
         self.not_end_frame = def_end_frame
-        
+
         if self.not_end_frame:
             # Call server method
             if hasattr (self, 'get_Frame_thread'):
@@ -628,7 +728,7 @@ class MainFrameRes(QtGui.QWidget):
         except:
             return None
         return None
-            
+
     def get_Frame_cycle(self, current_frame, through_object):
         if type(current_frame) == Exception:
             _print (_("Failed to get a frame from the server."))
@@ -652,7 +752,7 @@ class MainFrameRes(QtGui.QWidget):
                 if self.get_errno(current_frame) == 111:
                     sleeptime = 5
                 else:
-                    sleeptime = 0
+                    sleeptime = int(self.ClientObj.client.frame_period)
             else:
                 sleeptime = int(self.ClientObj.client.frame_period)
             self.get_Frame_thread = ClientServiceThread(self.ClientObj,
@@ -682,7 +782,7 @@ class MainFrameRes(QtGui.QWidget):
     def get_entire_frame(self, pid = None):
         """ get entire frame, from beginning (if client disconnected) """
         self.ClientObj.app.processEvents()
-    
+
         sid = get_sid(self.ClientObj.client)
         if not pid:
             try:
@@ -694,11 +794,11 @@ class MainFrameRes(QtGui.QWidget):
         else:
             list_pid = {}
             list_pid[0] = [pid]
-        
+
         if list_pid[0] == [0]:
             return 0
         pid = list_pid[0][0]
-        
+
         # Call server method
         if hasattr (self, 'get_Frame_thread'):
             if self.get_Frame_thread.isRunning():
@@ -759,7 +859,7 @@ class MainFrameRes(QtGui.QWidget):
         #    progressBar = self.add_Progress()
         #if self.progress_list:
         #    self.progress_list[-1].hide()
-        
+
         temp_progress = -1
         # Call server method
         if hasattr (self, 'get_Progress_thread'):
@@ -857,12 +957,13 @@ class MainFrameRes(QtGui.QWidget):
         self.show_table(table, item)
         self.ClientObj.app.processEvents()
         # Resize table
-        h = self.mytable.horizontalHeader().height() + \
-            2 * self.mytable.frameWidth()
-        h += self.mytable.horizontalScrollBar().sizeHint().height()
-        for row_in_table in range (self.mytable.rowCount()):
-            h += self.mytable.rowHeight(row_in_table)
-        self.mytable.setFixedHeight(h)
+        if self.mytable:
+            h = self.mytable.horizontalHeader().height() + \
+                2 * self.mytable.frameWidth()
+            h += self.mytable.horizontalScrollBar().sizeHint().height()
+            for row_in_table in range(self.mytable.rowCount()):
+                h += self.mytable.rowHeight(row_in_table)
+            self.mytable.setFixedHeight(h)
 
     def send_Confirm(self, sid, pid, item):
         """ send answer to the question """
@@ -946,11 +1047,11 @@ class MessageDialog(QtGui.QWidget):
         x = 1
         if isPassword:
             self.pass_edit.setEchoMode(self.pass_edit.Password)
-            if item.id == 2:        
+            if item.id == 2:
                 if hasattr(self,'pass_edit2'):
                     self.pass_edit2 = QtGui.QLineEdit(self)
                     self.pass_edit2.setEchoMode(self.pass_edit2.Password)
-                
+
                 self.layout.addWidget(LabelWordWrap(_('Repeat'), self), x, 0)
                 self.layout.addWidget(self.pass_edit2, x, 1)
                 x += 1
@@ -961,9 +1062,9 @@ class MessageDialog(QtGui.QWidget):
         self.send_button = QtGui.QPushButton(_('Send'), self)
         self.send_button.setShortcut(QtGui.QKeySequence(QtCore.Qt.Key_Return))
         self.send_button.clicked.connect(self.send_password)
-        
+
         self.layout.addWidget(self.send_button,x,1)
-                          
+
         self.move(parent.ClientObj._parent.geometry().x() + \
                   parent.ClientObj._parent.geometry().width() / 2 \
                   - self.sizeHint().width() / 2, \
@@ -972,12 +1073,12 @@ class MessageDialog(QtGui.QWidget):
                   - self.sizeHint().height() / 2)
         self.setWindowFlags (QtCore.Qt.WindowMinimizeButtonHint)
 #        self.setAttribute(QtCore.Qt.WA_ShowModal)
-        
+
     def send_password(self):
         answer = self.pass_edit.text()
         if not hasattr(self,"pass_edit2") or \
             answer == self.pass_edit2.text():
-            
+
             # Call server method
             if hasattr (self, 'send_password_thread'):
                 if self.send_password_thread.isRunning():
@@ -986,7 +1087,7 @@ class MessageDialog(QtGui.QWidget):
                             self._parent.ClientObj, 'send_message', self.sid, \
                             self.pid, answer, return_except = True)
             self.send_password_thread.signal.connect(self.send_password_after)
-            self.send_password_thread.start()            
+            self.send_password_thread.start()
         else:
             self.send_button.setDisabled(True)
 
